<?php namespace awp;

abstract class Module{
    protected $app;
    
    public function __construct()
    {
		$this->app = \awp\App::Instance();
    }

	protected function RequireMod($modName)
	{
        if( $this->app == null)
            $this->app = \awp\App::Instance();
		$this->app->RegisterModule($modName);
	}
}

//require_once "ConfigModule.php";

require_once "libs/utility.php";
require_once "libs/generate.php";
require_once "libs/strings.php";
require_once "classes/ConfigPage.php";

class App{

	private static $instance = null;
	private $modules;

	private $slug;
	private $version;
	private $cacheKey;
	private $cacheAllowed;
	
	
	private function __construct(){

		$this->slug = plugin_basename( __DIR__ );
		$this->version = AWP_VERSION;
		$this->cacheKey = 'ash-custom-upd';
		$this->cacheAllowed = false;

		$this->modules = array();
		
		add_filter( 'plugins_api', array( $this, 'Info' ), 20, 3 );
		add_filter( 'site_transient_update_plugins', array( $this, 'Update' ) );
		add_action( 'upgrader_process_complete', array( $this, 'Purge' ), 10, 2 );

		/*thanks https://wordpress.stackexchange.com/questions/314701/remove-view-details-option-from-plugin-listing */
		add_filter('plugin_row_meta',function ($plugin_meta, $plugin_file, $plugin_data, $status) {
			$pluginSlug = explode("/", $plugin_file)[0];
			$path = $_SERVER["REQUEST_SCHEME"] . "://" . $_SERVER["HTTP_HOST"] . "/wp-admin/" . (is_multisite()? "network/" : "");

			if($pluginSlug  == $this->slug)
				$plugin_meta[2] = "<a href=\"{$path}plugin-install.php?tab=plugin-information&plugin={$pluginSlug}&;TB_iframe=true\" class=\"thickbox open-plugin-details-modal\">View details</a>";

			return $plugin_meta;
		},10,4);

		add_action( 'admin_enqueue_scripts', function($page)
		{
			if($page == "plugin-install.php" && isset($_GET["plugin"]) && $_GET["plugin"] == dirname(plugin_basename( __FILE__ )))
            {
                wp_enqueue_style( "ash-wp-style", plugin_dir_url( __FILE__ ) . "ash-wp-style.css", array(), null);
				//wp_enqueue_style( "ash-wp-style-form", plugin_dir_url( __FILE__ ) . "styles/forms.css", array(), null);
            }
                

		});
	}
	
	//Called straight after instance is defined.
	private static function Initialise()
	{
			require_once("pages/config-page.php");
			require_once("pages/extras-page.php");
			require_once("pages/new-plugin-page.php");
			require_once("pages/user-options-page.php");
			$configPage = new ConfigEditorPage();
			$extrasPage =  new ExtrasPage($configPage);
			$newpluginPage =  new NewPluginPage($configPage);
			$userOptionsPage =  new UserOptionsPage($configPage);
	}

	public static function Instance()
	{
		if (self::$instance == null)
		{
			self::$instance = new app();
			self::Initialise();
		}
		
		return self::$instance;
	}

	public function Require($file, $once=true)
	{
		if(file_exists(__DIR__ . "/" .$file))
		{	
			if($once)
				require_once($file);
			else
				require($file);
		}
		else
		{
			
			$trace = debug_backtrace(0, 1)[0];
			echo "FILE:" . $trace["file"] . "<br/>";
			echo "LINE:" . $trace["line"] . "<br/>";
			echo "AT:" . $trace["function"] . "(" . implode(",", $trace["args"]) . ")<br/>";
			echo ("<br/>Could not find file {$file}.<br/>");
			
			
			die();
		}
	}

	public function RegisterModule($moduleName)
	{

		$className = str_replace('-', '', ucwords(strtolower($moduleName), '-'));
		//$str = lcfirst($str);
		$className = "\\awp\\".  $className;

		$moduleFile = "modules/" . strtolower($moduleName) . "-module.php";

		if(!file_exists(AWP_PATH . $moduleFile))
			throw new \ErrorException("could not find module named: '{$moduleName}'", 0);
	
		if(class_exists($className, false))
			return true;
	
		$success = require_once $moduleFile;
		$module = new $className();
		
		if(!is_subclass_of($module, "\\awp\\Module", false))
			throw new \ErrorException("Version must inherit from Module, for example 'class MyModule extends Module{}'", 0);

		if(!isset($this->modules[$moduleName]))
			$this->modules[$moduleName] = $module;
		
		return true;
	}

	public function __call( $method, $args )
	{
		if(!count($this->modules))
			throw new \ErrorException("No modules registerd, use \awp\App::Instance()->RegisterModule(\"moduleName\")", 0);

		$moduleNames = array_keys($this->modules);

		for($i=0; $i<count($moduleNames); $i++)
		{
			$moduleName = $moduleNames[$i];
			$classObj = $this->modules[$moduleName];
			if(method_exists($classObj, $method))
			{	
				return call_user_func_array(array($classObj, $method), $args);
			}
		}

		throw new \ErrorException("Method, $method, not found.", 0);
	}
	
	Private function Request(){
		$remote = get_transient( $this->cacheKey );
		
		if( false === $remote || ! $this->cacheAllowed ) 
		{
			$response = \request_page(AWP_UPDATE_URL, array(
				CURLOPT_TIMEOUT => 10,
				CURLOPT_HTTPHEADER => array( "Content-Type: application/json" ),
				CURLOPT_ENCODING => "",
				CURLOPT_HEADER => false,
				CURLOPT_RETURNTRANSFER => true,
				
			));
			
			if(	$response == null || $response["http_code"] !== 200 ||  empty($response["content"]) ) 
			{
				return false;
			}
			
			$remote = json_decode( $response['content'] );
			
			
			set_transient( $this->cacheKey, $remote, DAY_IN_SECONDS );
		}
		
		//prime download link
		\request_page($remote->download_url, array(
			CURLOPT_TIMEOUT => 10,
			CURLOPT_HTTPHEADER => array( "Content-Type: application/zip" ),
			CURLOPT_ENCODING => "",
			CURLOPT_HEADER => false,
			CURLOPT_RETURNTRANSFER => true,
		));
		
		return $remote;

	}
	

	public function Info($res, $action, $args )
	{

			// print_r( $action );
			// print_r( $args );

			// do nothing if you're not getting plugin information right now
			if( 'plugin_information' !== $action ) {
				return $res;
			}

			// do nothing if it is not our plugin
			if( $this->slug !== $args->slug ) {
				return $res;
			}

			// get updates
			$remote = $this->Request();

			if( ! $remote ) {
				return $res;
			}

			$res = new \stdClass();

			$res->name = $remote->name;
			$res->slug = $remote->slug;
			$res->version = $remote->version;
			$res->tested = $remote->tested;
			$res->requires = $remote->requires;
			$res->author = $remote->author;
			//$res->author_profile = $remote->author_profile;
			$res->download_link = $remote->download_url;
			$res->trunk = $remote->download_url;
			$res->requires_php = $remote->requires_php;
			$res->last_updated = $remote->last_updated;

			$res->sections = (array)$remote->sections;

			if( ! empty( $remote->banners ) ) {

				$res->banners = (array)$remote->banners;
			}

			return $res;
	}

	public function Update($transient)
	{
		
		if ( empty($transient->checked ) ) {
			return $transient;
		}

		$remote = $this->Request();

		if(
			$remote
			&& version_compare( $this->version, $remote->version, '<' )
			&& version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
			&& version_compare( $remote->requires_php, PHP_VERSION, '<' )
		) 
		{
			$res = new \stdClass();
			$res->slug = $this->slug;
			$res->plugin = "ash-wp/ash-wp.php";
			$res->new_version = $remote->version;
			$res->tested = $remote->tested;
			$res->package = $remote->download_url;

			$transient->response[ $res->plugin ] = $res;

		}

		return $transient;
	}

	public function Purge()
	{

		if (  $this->cacheAllowed && 'update' === $options['action'] && 'plugin' === $options[ 'type' ] ) {
			// just clean the cache when new plugin version is installed
			delete_transient( $this->cacheKey );
		}
	}


	
}