<?php namespace awp;
//TODO make a filter that lets you add custom input methods
//todo make a filter that lets you add handle input
class Form extends Module {

    public function __construct()
    {
        $this->inputFields = array(
            (object) [
                "name" => "text",
                "drawFunction" => function($input){ return $this->DrawSpecifiedInput("text", $input);},
                "checkFunction" => "text",
            ],
            (object) [
                "name" => "textarea",
                "drawFunction" => function($input){ return $this->DrawTextArea($input);},
                "checkFunction" => "text",
            ],
            (object) [
                "name" => "hidden",
                "drawFunction" => function($input){ return $this->DrawSpecifiedInput("hidden", $input);},
                "checkFunction" => "null",
            ],
            (object) [
                "name" => "toggle",
                "drawFunction" => function($input){ return $this->DrawTogleSwitch($input);},
                "checkFunction" => "bool",
            ],
            (object) [
                "name" => "array",
                "drawFunction" => function($input){ return $this->DrawListArea($input);},
                "checkFunction" => "textToArray",
            ],
            (object) [
                "name" => "number",
                "drawFunction" => function($input){ return $this->DrawSpecifiedInput("number", $input);},
                "checkFunction" => "number",
            ],
            (object) [
                "name" => "output",
                "drawFunction" => function($input){ return $this->DrawOuptput($input);},
                "checkFunction" => "null",
            ],
            (object) [
                "name" => "datetime",
                "drawFunction" => function($input){ return $this->DrawSpecifiedInput("datetime-local", $input);},
                "checkFunction" => "null",
            ],
            (object) [
                "name" => "date",
                "drawFunction" => function($input){ return $this->DrawSpecifiedInput("date", $input);},
                "checkFunction" => "null",
            ],
            (object) [
                "name" => "uuid",
                "drawFunction" => function($input){ return $this->DrawOuptput(\UUID($input));},
                "checkFunction" => "null",
            ],
            (object) [
                "name" => "dictionary",
                "drawFunction" => function($input){ return $this->DrawCustomInput($input, (object)["data-custom-input"=>"dictionary"]);},
                "checkFunction" => "dictionary",
            ],
        );


        $this->inputChecks = array(
            "text" =>  function($input){return $this->CheckString($input);},
            "number" =>  function($input){return $this->CheckNumber($input);},
            "textToArray" =>  function($input){return $this->CheckArrayString($input);},
            "bool" =>  function($input){return $this->CheckBool($input);},
            "boolean" => function($input){return $this->CheckBool($input);},
            "dictionary" => function($input){ return $this->CheckDictionary($input);},
            "null" => function($input){ return $input; },

        );

        add_action("wp_loaded", function(){
            $this->inputFields = apply_filters("ModifyInputFields", $this->inputFields);
            //echo "applied filter <br/>";
            //debug_print_backtrace(); echo "<br/>"; echo "<br/>";
            $this->inputChecks = apply_filters("ModifyInputChecks", $this->inputChecks);
        });

    }


    public function DrawInput($name, $item)
    {
        $field = $this->GetFieldConfig($name);
        call_user_func($field->drawFunction, (object)$item);
    }


    public function CheckInput($name, $input)
    {
        $field = $this->GetFieldConfig($name);

        $callback =  array_key_exists($field->checkFunction, $this->inputChecks) ? $this->inputChecks[$field->checkFunction] : null;


        return call_user_func($callback, $input);
    }

    
    private function GetFieldConfig($name)
    {
        $debugNames = [];
        for($i=0; $i<count($this->inputFields); $i++)
        {
            $current = is_array($this->inputFields[$i]) ? (object)$this->inputFields[$i] : $this->inputFields[$i];
            $debugNames[] = $current->name;

            if($current->name == $name)
            {
                //print_r($this->inputFields[$i]->name);echo"<br/>";
                return $current;
            }
        }

        //debug_print_backtrace(0, 6);echo "<br/>";
        echo "Filed '{$name}' doesn't exist.<br/>Try one of these: "; print_r($debugNames); echo ".";
    }

    private static function DrawTogleSwitch($item)
    {
        $setting = get_option($item->setting, array());
        $setting = empty($setting) ? array() : $setting;
        $value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default;

        ?>
        <div>
            <h3><?=$item->label?></h3>
            <input type="hidden" name="<?=$item->setting?>[<?=$item->key?>]" value=false />
            <label class="switch">
                <input type="checkbox" value="true" name="<?=$item->setting?>[<?=$item->key?>]" <?=checked($value)?> />
                <span class="slider"></span>
            </label>
            <p class="description"><?=$item->description?></p>
        </div>
        <?php
    }

    private static function DrawListArea($item)
    {
        $setting = get_option($item->setting, array());
        $setting = empty($setting) ? array() : $setting;
        $value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default;

        ?>
        <div class="list-area">
            <h3><?=$item->label?></h3>
            <textarea name="<?=$item->setting?>[<?=$item->key?>]"><?=is_array($value) ? implode("\n", $value) : $value?></textarea>
        </div>
        <?php
    }

    private static function DrawTextArea($item)
    {
        $setting = get_option($item->setting, array());
        $setting = empty($setting) ? array() : $setting;
        $value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default;

        ?>
        <div>
            <h3><?=$item->label?></h3>
            <textarea name="<?=$item->setting?>[<?=$item->key?>]"><?=$value?></textarea>
        </div>
        <?php
    }

    private static function DrawOuptput($item)
    {
        $setting = get_option($item->setting, array());
        $setting = empty($setting) ? array() : $setting;
        $value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default ;

        ?>
        <div>
            <h3><?=$item->label?></h3>
            <output name="<?=$item->setting?>[<?=$item->key?>]" ><?=$value?></output>
        </div>
        <?php
    }

    private static function DrawSpecifiedInput($type, $item)
    {
        $setting = get_option($item->setting, array());
        $setting = empty($setting) ? array() : $setting;
        $value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default ;

        ?>
        <div> 
            <?php if($type != "hidden"): ?>
            <h3><?=$item->label?></h3>
            <?php endif; ?>
            <input type="<?=$type?>" value="<?=$value?>" name="<?=$item->setting?>[<?=$item->key?>]"  />
        </div>
        <?php
    }
    
    private static function DrawCustomInput($item, $attributes = [])
    {
        $setting = get_option($item->setting, array());
        $setting = empty($setting) ? array() : $setting;
        
        $value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default ;
        
        if(is_array($attributes))
            $attributes = (object)$attributes;
        
        $attributesOutput = "";
        $keys = array_keys(get_object_vars($attributes));
        
        for($i=0; $i<count($keys); $i++)
        {
            $key = $keys[$i];
            $attr = $attributes->$key;
            $attributesOutput .= "{$key}=\"{$attr}\"";
        }
        
        ?>
        <div> 
            <h3><?=$item->label?></h3>
            <input value='<?=$value?>' name="<?=$item->setting?>[<?=$item->key?>]" <?=$attributesOutput?> />
        </div>
        <?php
    }
    
    private function CheckString($input)
    {
        return sanitize_text_field($input);
    }

    private function CheckNumber($input)
    {
        return floatval($input);
    }

    private function CheckArrayString($input)
    {
        if(is_array($input))
            return $input;

		$newValue = $input;
		$newValue = sanitize_textarea_field($newValue);
		$newValue = explode("\n", $newValue);
		$newValue = array_map('trim', $newValue);

		return $newValue;
    }

    private function CheckBool($input)
    {
        return json_decode($input);
    }
    
    private function CheckDictionary($input)
    {
        $data = json_decode($input) ?? array();
        $output =(object)[];
        for($i=0; $i<count($data); $i++)
        {
            $key = $data[$i]->key;
            $value = $data[$i]->value;
            
            $output->$key = $value;
        }
        
        return json_encode($output);
    }
}


?>