<?php namespace awp;

class PageLoading extends Module{
    public function __construct(){
        
        add_action( "DisableTagOptions", array($this, "DisableTagOptions") );
        $this->EnableTagOptions();
        
		add_action( "RenderPageTemplates", array($this, "RenderPageTemplates") );
        
        //var_dump("pageLoading constructor");
        //print_r( debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 6));
        //echo "<br/></br>";
        
        add_action("wp_enqueue_scripts", function() { do_action("site_enqueue_scripts"); });
        add_action("admin_enqueue_scripts", function() { do_action("site_enqueue_scripts"); });

    }
    
    public function DoSitesScripts()
    {
        
    }
    

    public function EnableTagOptions()
    {
        add_filter( "style_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
        add_filter( "script_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
    }

    public function DisableTagOptions()
    {
        remove_filter( "style_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
        remove_filter( "script_loader_tag", array($this, "CustomTagOptions"), 10, 3 );
    }

    public function CustomTagOptions($tag, $handle, $src)
    {
        //Find all attributes marked with custom tags, if none continue as norma.
        $customTags = ["%5B", "%5D"]; // URL SAFE VERSION OF [ AND ]
        $attRegEx = "/{$customTags[0]}(.*)(?:(=)(.*))?{$customTags[1]}/U";
        $matches = [];
        if(!preg_match_all ($attRegEx, $src, $matches, PREG_UNMATCHED_AS_NULL|PREG_SET_ORDER ))
                return $tag;

        //Grab the tag name and the close tag+method*. *could be ></script> OR /> or >
        $tagEnds;
        $tagRegEx = "/^<(?<tag>\w+).*?(>[^>]*<\/\k'tag'>|\/>|>)$/";
        preg_match($tagRegEx ,$tag, $tagEnds);
        
        //stringyfy new attributes and replace custom tags with new string.
        $attributesString = "";
        $changeType = false;
        for($i=0; $i<count($matches); $i++)
        {
            $src = str_replace ($matches[$i][0], "", $src);
            $attributesString .= " {$matches[$i][1]}" . (isset($matches[$i][3]) ? "{$matches[$i][2]}\"" . trim($matches[$i][3], "'\"`‘’") . "\"" : "");
            if(strtolower($matches[$i][1]) == "type")
                $changeType = true;
        }

        //build new tag and add default js type to make code less complainy.
        return "<{$tagEnds[1]} " . (!$changeType && $tagEnds[1]=="script" ? "type=\"text/javascript\" " : "") . ($tagEnds[1]=="script" ? "src": "href") . "=\"" . esc_url($src) ."\"{$attributesString}{$tagEnds[2]}\n";
    }


    public function RenderPageTemplates()
    {
        add_filter("template_include", array($this, "UnblockPageTemplate"), 10, 3 );
    }

    public function RemovePageTemplates()
    {
        remove_filter("template_include", array($this, "UnblockPageTemplate"), 10, 3 );
    }

    public function UnblockPageTemplate($template)
    {

        if(!is_404())
            return $template;

        global $wp, $wp_query;
        $request = stripslashes(urldecode($wp->request));
        
        $safeRequest;
        if(!preg_match("/^[\w\d\-_.?\/# ]+$/", $request, $safeRequest, PREG_UNMATCHED_AS_NULL))
            return $template;

        $safeRequest = explode("/", $safeRequest[0]);
        /*
        add_filter("PageMaxRequestSections", function($requ){
            return 4; // or more
        });*/
        
        $pageMaxRequestSections = apply_filters("PageMaxRequestSections", $safeRequest);
        
        
        $pageMaxRequestSections  = $pageMaxRequestSections == $safeRequest ? 0 : intval($pageMaxRequestSections);

        
	    $sectionCheck = substr_count($request,"/") <= (int)$pageMaxRequestSections;
        
        if(!$sectionCheck)
            return $template;

        if($pageMaxRequestSections > 0)
        {
            $data = array();
            while(next($safeRequest) !== false)
            {
                $data[] = current($safeRequest);
            }
            
            reset( $safeRequest );
            set_query_var("request_sections", $data);
            //get_query_var("request_sections");
        }
        
        
        $safeRequest = $safeRequest[0];
        $file = get_stylesheet_directory() . "/page-" . $safeRequest . ".php";
        if(file_exists($file))
        {
            status_header( 200 );
			$wp_query->is_page = true;
			$wp_query->is_404 = false;
            
            
            add_filter('wp_title', function($title) use($safeRequest){
                if(empty($title))
                    $title = ucwords($safeRequest, "-");
                
                return $title;
            });


            return $file;

        }


        return $template;
    }

    public function Load404()
    {
        add_action( 'wp_head', function(){
            ?>
            <meta name="robots" content="noindex, follow" />
            <?php
        },2 );

        global $wp_query;
        $wp_query->set_404();
        status_header( 404 );
        get_template_part( 404 ); 

        //
        exit();
    }


}