<?php namespace awp;
class UserOptionsPage extends ConfigPage
{
	public function __construct($parent)
	{
        
		$this->app = \awp\App::Instance();
		$this->RequireMod("cleanup");
		$this->RequireMod("page-loading");
		$this->RequireMod("debug");


        parent::__construct(
            array("parent"=> $parent, "slug"=>"ash-wp-user-options", "title"=> "User Options", "tabTitle"=>"User Options"),
            array("style-forms", "style-buttons", "style-cards")
        );
        
        //Fire before any other plugin as theses settings may be required in the other plugins.
		add_action("plugins_loaded", function(){ $this->ApplySettings(); }, -1 );
		
		

	}
	
	
	protected function Begin()
	{
		$settings = get_user_meta(get_current_user_id(), "awp-user-options", true);
		$settings = empty($settings) ? array() : $settings;
		
		$displayErrors = array_key_exists("displayErrors", $settings) ? $settings["displayErrors"] : WP_DEBUG_DISPLAY ;
		$depricationWarings = array_key_exists("depricationWarings", $settings) ? $settings["depricationWarings"] : true ;
		$fabricateError = array_key_exists("fabricateError", $settings) ? $settings["fabricateError"] : false ;
		

        $debugFields = [
            "displayErrors" => ["label"=>"Display  Errors", "default"=>$displayErrors, "type"=>"toggle", 
				"description"=>"Enables error messages on your user only."],
            "depricationWarings" => ["label"=>"Show Deprication Warings", "default"=>$depricationWarings, "type"=>"toggle", 
				"description"=>"Displays deprication warnigns by default, you may want to disable this after a PHP update."],
            "fabricateError" => ["label"=>"Fabricate Error", "default"=>$fabricateError, "type"=>"toggle", 
				"description"=>"Outputs a false error message to show you where the the error is being loged/displayed."],

        ];

        
		$formData = array(
			"ash-wp-users-config-form" => array(
				"ash-wp-user-debug" => array_merge(
					array("groupTitle" => "Debug Options"),
					$debugFields,
				),				"submit" => array(
					"text" => "",
					"action" => "user-options",
					"settingName" => "ash-wp-user-settings",
				)
			)
		);
		
		$this->AddComponent($formData);
	}

	/**
	 * Called before admin page is loaded
	 **/ 
	protected function Load()
	{

	}

	/**
	 * Called when data is posted.
	 **/ 
	protected function Request($sanitisedInput)
	{
		$settings = get_user_meta(get_current_user_id(), "awp-user-options", true);
		
		$settings = $settings == null ? array() : $settings;
		
		$settings["displayErrors"] = $sanitisedInput->displayErrors;
		$settings["depricationWarings"] = $sanitisedInput->depricationWarings;
		$settings["fabricateError"] = $sanitisedInput->fabricateError;
		
		update_user_meta(get_current_user_id(), "awp-user-options", $settings);
		
		$this->RedirectBack();
	}
	
	public function ApplySettings(){
		
		
		$settings = get_user_meta(get_current_user_id(), "awp-user-options", true);
		$settings = empty($settings) ? array() : $settings;
		
		$displayErrors = array_key_exists("displayErrors", $settings) ? $settings["displayErrors"] : WP_DEBUG_DISPLAY ;
		$displayErrors = $displayErrors ? true : false;
		
		$depricationWarings = array_key_exists("depricationWarings", $settings) ? $settings["depricationWarings"] : true;
		
		$fabricateError = array_key_exists("fabricateError", $settings) &&  $settings["fabricateError"] == true;
		
		if($displayErrors)
		{
			//wp_debug_mode();
			ini_set( 'display_errors', 1 );
			
		}
		
		if(!$depricationWarings)
		{
			error_reporting(E_ALL & ~E_DEPRECATED);
			/*
			add_filter( 'deprecated_constructor_trigger_error', '__return_false' );
			add_filter( 'deprecated_function_trigger_error', '__return_false' );
			add_filter( 'deprecated_file_trigger_error', '__return_false' );
			add_filter( 'deprecated_argument_trigger_error', '__return_false' );
			add_filter( 'deprecated_hook_trigger_error', '__return_false' );
			*/
		}
		
		if($fabricateError)
		{
			$hash = \HexString(2);
			trigger_error("This is fabricate warning to demonstrate a test! Random Hex: ({$hash})");
		}
		
		//$this->app->SetConfigDefined("WP_DEBUG_DISPLAY", $displayErrors ? true : false);
		
		
		//throw new \Exception('Division by zero.');

	}




}


